# -*- coding: utf-8 -*-
from odoo.addons.saas_worker.util import version_gte
from odoo.addons.saas_worker.config import saas_config

try:
    import sentry_sdk
except ImportError:
    sentry_sdk = False
SENTRY_DSN_KEY = saas_config.get('trial', {}).get('sentry_dsn_key')

if version_gte("saas~16.4") and sentry_sdk and SENTRY_DSN_KEY:

    from lxml import etree
    from odoo import models, SUPERUSER_ID
    from odoo.addons.base.models.ir_qweb import QWebException

    class Base(models.AbstractModel):
        _inherit = 'base'

        def _should_be_sentry_ignored(self):
            # Suppose `self._cr` is unusable because of a `DatabaseError`. Then,
            # if we query the DB, it will raise a `InFailedSqlTransaction` error.
            # We have to avoid such a situation.
            with self.env.registry.cursor() as cr:
                self = self.with_env(self.env(cr=cr, user=SUPERUSER_ID))
                return any(record.write_uid.id != SUPERUSER_ID for record in self)

    class IrActionsServer(models.Model):
        _inherit = 'ir.actions.server'

        def run(self):
            try:
                res = super().run()
            except Exception as e:
                e.sentry_ignored = self._should_be_sentry_ignored()
                raise
            return res

    class IrView(models.Model):
        _inherit = 'ir.ui.view'

        def _raise_view_error(self, message, node=None, *, from_exception=None, from_traceback=None):
            try:
                super()._raise_view_error(message, node=node, from_exception=from_exception, from_traceback=from_traceback)
            except Exception as e:
                views = self._get_children_and_parents()
                e.sentry_ignored = views._should_be_sentry_ignored()
                raise

        def _get_children_and_parents(self):
            view = self.sudo()
            while view.inherit_id:
                view = view.inherit_id
            return view.search([('inherit_id', 'child_of', view.id)])

    qweb_exception_init = QWebException.__init__
    def saas_init(self, message, qweb, template=None, ref=None, path_xml=None, code=None):
        if isinstance(ref, int) and not isinstance(template, etree._Element):
            template = qweb.env['ir.ui.view'].browse(ref).exists()
            templates = template._get_children_and_parents()
            self.sentry_ignored = templates._should_be_sentry_ignored()
        return qweb_exception_init(self, message, qweb, template=template, ref=ref, path_xml=path_xml, code=code)
    QWebException.__init__ = saas_init
